/* 
   sitecopy, remote web site copy manager.
   Copyright (C) 1998, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* This is the console front end for sitecopy.
 * TODO: Some of the configuration stuff should be moved out of here. */

#include <sys/param.h>

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include "console_fe.h"
#include "frontend.h"
#include "sites.h"
#include "common.h"

/* The defined sites from sites.c */
extern struct site_t *all_sites;

/* Settings the debugging level (common.c) */
extern enum debug_t debug_level;

#define RCNAME "/." TARGET "rc"
#define COPYNAME "/." TARGET "/"

/* Kludgy stuff - this returns whether the given action requires
   that a site name is specified in the cmdline */
#define NEEDSITE(x) (x==action_list || x==action_update || \
    (x==action_catchup) || (x==action_resync) || (x==action_init))
/* The maximum number of sites which can be specified on the command
 * line */

#define MAXSITES 20

enum action_t action;

/* Misc global vars */
char *progname; /* argv[0] */
char *rcfile; /* filename of rcfile */
char *copypath; /* path to info file storage directory */

char *sitenames[MAXSITES]; /* the sites specified on the command line */
int numsites; /* the number of sites specified */

/* Misc options */
bool prompting; /* Do we want confirmations? */
bool allsites; /* Do they want all sites to be operated on? */
int quiet; /* How quiet do they want us to be? */

int site_list( struct site_t *the_site ) {
    struct site_file_t *current;
    int count;
    if( the_site->numnew > 0 ) {
	printq( 0, "* These items have been added since the last update:\n" );
	count = 0;
	for( current = the_site->files; current!=NULL; current=current->next) {
	    if( current->diff == file_new ) {
		if( count++ ) printq( 0, ", " );
		if( current->dir ) printq( 0, "dir:" );
		printq( 0, "%s", current->rel_local);
	    }
	}
	printq( 0,"\n");
    }
    if( the_site->numchanged > 0 ) {
	printq( 0, "* These items have been changed since the last update:\n" );
	count = 0;
	for( current = the_site->files; current!=NULL; current=current->next) {
	    if( current->diff == file_changed ) {
		if( count++ ) printq( 0, ", " );
		if( current->dir ) printq( 0, "dir:" );
		printq( 0, "%s", current->rel_local);
	    }
	}
	printq( 0,"\n");
    }
    if( the_site->numdeleted > 0 ) {
	printq( 0, "* These items have been deleted since the last update:\n" );
	count = 0;
	for (current = the_site->files; current!=NULL; current=current->next) {
	    if( current->diff == file_deleted ) {
		if( count++ ) printq( 0, ", " );
		if( current->dir ) printq( 0, "dir:" );
		printq( 0, "%s", current->rel_local );
	    }
	}
	if( the_site->nodelete ) {
	    printq( 0, "-- These files will be left on the server --\n" );
	}
	printq( 0,"\n");
    }
    if( the_site->nummoved > 0 ) {
	printq( 0, "* These items have been moved since the last update:\n" );
	count = 0;
	for (current = the_site->files; current!=NULL; current=current->next) {
	    if( current->diff == file_moved ) {
		if( count++ ) printq( 0, ", " );
		printq( 0, "%s->/%s", current->old->rel_local, 
			current->directory  );
	    }
	}
	printq( 0,"\n");
    }
    return 0;
}

void site_dumpsites( ) {
    struct site_t *current;
    for( current=all_sites; current!=NULL; current=current->next ) {
	/* FIXME: Possibly, make sure we have the actual port number first
	 * so we don't have to mess around printing out the port */
	printf( "Site: %s\n"  "\tServer: %s", current->name, current->server );
	if( current->port == 0 ) {
	    printf( "  Port: %s\n", current->driver->service_name );
	} else {
	    printf( "  Port: %d\n", current->port );
	}
	printf( "\tProtocol: %s    Username: %s\n"
		"\tRemote directory: %s%s\n"
		"\tLocal directory: %s\n"
		"\tPermissions: %s     Symlinks: %s\n", 
		current->driver->protocol_name, current->username, 
		DIRPREFIX(current->remote_root),
		current->remote_root, current->local_root, (current->perms==sitep_ignore)?"Ignored":(current->perms==sitep_exec)?"Maintained for executables":"Always maintained", (current->symlinks==sitesym_ignore?"Ignored":"Followed"));
	if( current->nodelete ) printq( 0, "\tRemote files will not be deleted.\n" );
	if( current->checkmoved ) printq( 0, "\tFiles will be moved remotely if moved locally.\n" );
    }
}

void init( int argc, char *argv[] ) {
    char *home;
    int optc;
    extern char *optarg;
    extern int optind;
    
    /* Defaults */
    allsites = prompting = false;
    action = action_list;
    prompting = false;
    debug_level = debug_none;
    progname = basename( argv[0] );

    /* Read the cmdline args */
    while( (optc = getopt( argc, argv, "d:p:r:hucilvya" ) ) != -1) {
	switch( optc ) {
	case 'l': action = action_list; break;
	case 's': action = action_resync; break;
	case 'q': quiet++; break;
	case 'r': 
	    if( strlen(optarg) != 0 ) {
		rcfile = strdup( optarg );
	    } else {
		usage( );
		exit(-1);
	    }
	    break;
	case 'p':
	    if( strlen(optarg) != 0 ) {
		if( optarg[strlen(optarg)] != '/' ) {
		    copypath = malloc( strlen(optarg) + 2 );
		    strcpy( copypath, optarg );
		    strcat( copypath, "/" );
		} else {
		    copypath = strdup( optarg );
		}
	    } else {
		usage( );
		exit(-1);
	    }
	    break;
	case 'i': action = action_init; break;
	case 'c': action = action_catchup; break;
	case 'u': action = action_update; break;
	case 'v': action = action_view; break;
	case 'd': /* set debugging level */
	    switch( atoi(optarg) ) {
	    case 0: 
		debug_level = debug_none;
		printq( 0, "Debugging off.\n" );
		break;
	    case 1:
		debug_level = debug_some;
		printq( 0, "Debugging some.\n" );
		break;
	    case 2: 
		debug_level = debug_full; 
		printq( 0, "Debugging full.\n" );
		break;
	    case 3:
		debug_level = debug_total;
		printq( 0, "Really excessive amounts of debugging.\n" );
		break;
	    default:
		usage( ); exit(-1);
	      }
	    break;
	case 'y':
	    prompting = true;
	    printq( 0, "Prompting on.\n" );
	    break;
	case 'a':
	    allsites = true;
	    break;
	case 'h': usage( ); exit(-1);
	case '?': usage( ); exit(-1);
	}
    }
    
    /* Get those site names off the end of the cmdline */
    for( numsites = 0 ; optind < argc; optind++ ) {
      sitenames[numsites++] = argv[optind];
      if( numsites == MAXSITES ) {
	debug( debug_none, "%s: Warning: Only %d sites can be specified on the command line!\nExtra entries are being skipped.\n", progname, MAXSITES );
	    break;
	}
    }

    /* Assign default filenames if they didn't give us any */
    home = getenv("HOME");
    if( rcfile == NULL ) {
	rcfile = (char *) malloc( strlen(RCNAME) + strlen(home) + 1);
	strcpy( rcfile, home );
	strcat( rcfile, RCNAME );
    }
    if( copypath == NULL ) {
	copypath = (char *) malloc( strlen(COPYNAME) + strlen(home) + 1);
	strcpy( copypath, home );
	strcat( copypath, COPYNAME );
    }
}

/* Prompts for confirmation - returns */
bool fe_ynprompt( char *msg ) {
    char in[BUFSIZ];
    printf( "%s? (y/n) ", msg );
    fgets( in, BUFSIZ, stdin );
    return (*in == 'y'); /* only return true if they put a 'y' first */
}

void fe_status( char *msg ) {
    printf( msg );
}

void fe_error( char *msg ) {
    printf( msg );
}

void printq( int q, char *template, ...) {
    va_list params;
/*    if( q==0 || quiet!=2 ) { */
        va_start( params, template );
        vprintf( template, params );
        va_end( params ) ;
/*     } */
}

int main( int argc, char *argv[] ) {
    int ret, num, numgoodsites;
    struct site_t *current;

    /* Initialize up */
    ret = 0;
    init( argc, argv );
    
    /* Signal handling shtuff.
     * Ignore 'broken pipe' signals.
     signal( SIGPIPE, SIG_IGN ); 
     * nah, better to handle socket io properly - jo */
    
    /* Read the rcfile */
    if( ( ret = site_readsites( rcfile ) ) ) {
	printf( "%s: Error: Could not read rcfile: %s.\n",
		 progname, rcfile );
	exit(-1);
    }

    if( NEEDSITE( action ) ) {
	/* Find the site they want */
        
	numgoodsites = numsites;

	for( num=0; num < numsites; num++ ) {
	    if( (ret = site_find( sitenames[num], &current )) ) { 
		numgoodsites--;
		switch( ret ) {
		case SITE_NOSITE:
		    printf( "%s: Error: No site called `%s' found - skipping.\n", 
			    progname, sitenames[num] );
		case SITE_NONAME:
		    printf( "%s: Site name not specified.\n", progname );
		    break;
		case SITE_ACCESSLOCALDIR:
		    printf( "%s: Could not read directory for `%s':\n\t%s\n", 
			    progname, sitenames[num], current->local_root );
		    break;
		case SITE_NOSERVER:
		    printf( "%s: Server not specified in site `%s'.\n", progname, sitenames[num] );
		    break;
		case SITE_NOUSERNAME:
		    printf( "%s: Username not specified in site `%s'.\n", progname, sitenames[num] );
		    break;
		case SITE_NOPASSWORD:
		    printf( "%s: Password not specified in site `%s'.\n", progname, sitenames[num] );
		    break;
		case SITE_NOREMOTEDIR:
		    printf( "%s: Remote directory not specified in site `%s'.\n", progname, sitenames[num] );
		    break;
		case SITE_NOLOCALDIR:
		    printf( "%s: Local directory not specified in site `%s'.\n", progname, sitenames[num] );		   
		}
	    } else if( site_readfiles( current ) ) {
		printq( 0, "%s: Error: Could not read site info file for `%s'.\n",
			progname, sitenames[num] );
		numgoodsites--;
	    } else {
		/* Got a valid site! */
		current->use_this = true;
	    }
	}
	
	if( numgoodsites==0 ) {
	    printq( 0, "%s: No sites specified.\n", progname );
	    exit(-1);
	}
		
    }

    /* Handle the dump sites request */
    if( action == action_view ) {
	site_dumpsites( );
	return 0;
    }

    for( current=all_sites; current!=NULL; current=current->next ) {
	
	if( !current->use_this && !allsites) 
	    continue;

	printf( "%s: Site `%s' (%s:%s%s) in %s\n", 
		progname, current->name, current->server, 
		DIRPREFIX(current->remote_root), current->remote_root,
		current->local_root );
	
	switch( action ) {
	case action_update:
	    if( current->has_changed ) {
		ret = site_update( current );
	    } else {
		printq( 0, "%s: Nothing to update - no changes found.\n", progname );
		ret = 0;
	    }
	    break;
	case action_list:
	    site_list( current );
	    if( current->has_changed ) {
		printq( 0, "%s: The remote site needs updating.\n", progname);
		ret = 1;
	    } else {
		printq( 0, "%s: The remote site does not need updating.\n", progname );
		ret = 0;
	    }
	    break;
	case action_init:
	    printq( 0,"Initializing site... " );
	    site_initialize( current );
	    site_writefiles( current );
	    printq( 0, "done.\nAll the files and directories are marked as NOT updated remotely.\n" );
	    break;
	case action_catchup:
	    printq( 0, "Catching up site... " );
	    site_catchup( current );
	    site_writefiles( current );
	    printq( 0, "done.\n"
		    "All the files and and directories are marked as updated remotely.\n" );
	    break;
	default:
	    printq( 0,"Unimplemented - sorry.\n");
	    ret = -1;
	    break;
	}
    }
    
    return ret;
}

void usage( ) {
    fprintf( stderr,
"Usage: %s [options] sitename [sitename...]\n"
"Options: [-d x] {-h|-v|-l|-i|-c|-u} [-y] [-r fn] [-p dir]\n"
"\n"
"\t-d\tSet debugging level - 0=no debugging...3=excessive debugging\n"
"\t-r\tSpecify alternate run control file\n"
"\t-p\tSpecify alternate site storage directory\n"
"\t-y\tRequest confirmation before making each update\n"
"\t-h\tDisplay this help message\n"
"\t-v\tList the site definitions\n"
"\t-l\tList files which have been added, changed, moved and deleted\n"
"\t-i\tInitialize - mark all files and directories as not updated\n"
"\t-c\tCatch up - mark all files and directories as updated\n"
"\t-u\tUpdate the remote site\n\n"
"I am " TARGET " version %s, console user interface.\n"
"\tProtocols supported: FTP"
#ifdef USE_DAV
", HTTP-DAV"
#endif
"\n\n", progname, VERSION );

/* \t-a\tPerform the given action on ALL sites defined\n\
   \t-r\tResyncronize the remote site list\n\
*/

}
